@@ -334,19 +334,6 @@
   "paddw      %[ftmp3],   %[ftmp3],       %[ftmp1]            \n\t"
 #endif /* _MIPS_SIM == _ABIO32 */
 
-// depending on call sites, pass **ref_array to avoid & in subsequent call and
-// de-dup with 4D below.
-#define sadMxNxK_mmi(m, n, k)                                                 \
-  void vpx_sad##m##x##n##x##k##_mmi(const uint8_t *src, int src_stride,       \
-                                    const uint8_t *ref_array, int ref_stride, \
-                                    uint32_t *sad_array) {                    \
-    int i;                                                                    \
-    for (i = 0; i < k; ++i)                                                   \
-      sad_array[i] =                                                          \
-          vpx_sad##m##x##n##_mmi(src, src_stride, &ref_array[i], ref_stride); \
-  }
-
-// This appears to be equivalent to the above when k == 4 and refs is const
 #define sadMxNx4D_mmi(m, n)                                                  \
   void vpx_sad##m##x##n##x4d_mmi(const uint8_t *src, int src_stride,         \
                                  const uint8_t *const ref_array[],           \
@@ -366,16 +353,16 @@ static inline unsigned int vpx_sad64x(const uint8_t *src, int src_stride,
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
+    "pxor       %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_REF_ABS_SUB_64
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_REF_ABS_SUB_64
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp5]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
@@ -407,27 +394,28 @@ static inline unsigned int vpx_sad_avg64x(const uint8_t *src, int src_stride,
   unsigned int sad;
   double ftmp1, ftmp2, ftmp3, ftmp4, ftmp5;
   mips_reg l_counter = counter;
+  mips_reg l_second_pred = (mips_reg)second_pred;
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
+    "pxor       %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_AVGREF_ABS_SUB_64
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x40            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x40)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_AVGREF_ABS_SUB_64
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x40            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x40)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp5]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
       [ftmp4]"=&f"(ftmp4), [ftmp5]"=&f"(ftmp5), [counter]"+&r"(l_counter),
       [src]"+&r"(src), [ref]"+&r"(ref),
-      [second_pred]"+&r"((mips_reg)second_pred),
+      [second_pred]"+&r"(l_second_pred),
       [sad]"=&r"(sad)
     : [src_stride]"r"((mips_reg)src_stride),
       [ref_stride]"r"((mips_reg)ref_stride)
@@ -456,16 +444,16 @@ static inline unsigned int vpx_sad32x(const uint8_t *src, int src_stride,
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
+    "pxor       %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_REF_ABS_SUB_32
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_REF_ABS_SUB_32
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp5]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
@@ -499,27 +487,28 @@ static inline unsigned int vpx_sad_avg32x(const uint8_t *src, int src_stride,
   unsigned int sad;
   double ftmp1, ftmp2, ftmp3, ftmp4, ftmp5;
   mips_reg l_counter = counter;
+  mips_reg l_second_pred = (mips_reg)second_pred;
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
+    "pxor       %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_AVGREF_ABS_SUB_32
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x20            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x20)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_AVGREF_ABS_SUB_32
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x20            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x20)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp5]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
       [ftmp4]"=&f"(ftmp4), [ftmp5]"=&f"(ftmp5), [counter]"+&r"(l_counter),
       [src]"+&r"(src), [ref]"+&r"(ref),
-      [second_pred]"+&r"((mips_reg)second_pred),
+      [second_pred]"+&r"(l_second_pred),
       [sad]"=&r"(sad)
     : [src_stride]"r"((mips_reg)src_stride),
       [ref_stride]"r"((mips_reg)ref_stride)
@@ -549,16 +538,16 @@ static inline unsigned int vpx_sad16x(const uint8_t *src, int src_stride,
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
+    "pxor       %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_REF_ABS_SUB_16
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_REF_ABS_SUB_16
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp5]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
@@ -581,10 +570,6 @@ static inline unsigned int vpx_sad16x(const uint8_t *src, int src_stride,
 vpx_sad16xN(32);
 vpx_sad16xN(16);
 vpx_sad16xN(8);
-sadMxNxK_mmi(16, 16, 3);
-sadMxNxK_mmi(16, 16, 8);
-sadMxNxK_mmi(16, 8, 3);
-sadMxNxK_mmi(16, 8, 8);
 sadMxNx4D_mmi(16, 32);
 sadMxNx4D_mmi(16, 16);
 sadMxNx4D_mmi(16, 8);
@@ -596,27 +581,28 @@ static inline unsigned int vpx_sad_avg16x(const uint8_t *src, int src_stride,
   unsigned int sad;
   double ftmp1, ftmp2, ftmp3, ftmp4, ftmp5;
   mips_reg l_counter = counter;
+  mips_reg l_second_pred = (mips_reg)second_pred;
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
+    "pxor       %[ftmp5],   %[ftmp5],       %[ftmp5]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_AVGREF_ABS_SUB_16
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x10            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x10)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_AVGREF_ABS_SUB_16
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x10            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x10)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp5]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
       [ftmp4]"=&f"(ftmp4), [ftmp5]"=&f"(ftmp5), [counter]"+&r"(l_counter),
       [src]"+&r"(src), [ref]"+&r"(ref),
-      [second_pred]"+&r"((mips_reg)second_pred),
+      [second_pred]"+&r"(l_second_pred),
       [sad]"=&r"(sad)
     : [src_stride]"r"((mips_reg)src_stride),
       [ref_stride]"r"((mips_reg)ref_stride)
@@ -646,16 +632,16 @@ static inline unsigned int vpx_sad8x(const uint8_t *src, int src_stride,
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
+    "pxor       %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_REF_ABS_SUB_8
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_REF_ABS_SUB_8
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp3]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
@@ -678,10 +664,6 @@ static inline unsigned int vpx_sad8x(const uint8_t *src, int src_stride,
 vpx_sad8xN(16);
 vpx_sad8xN(8);
 vpx_sad8xN(4);
-sadMxNxK_mmi(8, 16, 3);
-sadMxNxK_mmi(8, 16, 8);
-sadMxNxK_mmi(8, 8, 3);
-sadMxNxK_mmi(8, 8, 8);
 sadMxNx4D_mmi(8, 16);
 sadMxNx4D_mmi(8, 8);
 sadMxNx4D_mmi(8, 4);
@@ -693,26 +675,27 @@ static inline unsigned int vpx_sad_avg8x(const uint8_t *src, int src_stride,
   unsigned int sad;
   double ftmp1, ftmp2, ftmp3;
   mips_reg l_counter = counter;
+  mips_reg l_second_pred = (mips_reg)second_pred;
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
+    "pxor       %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_AVGREF_ABS_SUB_8
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x08            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x08)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_AVGREF_ABS_SUB_8
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x08            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x08)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp3]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
       [counter]"+&r"(l_counter), [src]"+&r"(src), [ref]"+&r"(ref),
-      [second_pred]"+&r"((mips_reg)second_pred),
+      [second_pred]"+&r"(l_second_pred),
       [sad]"=&r"(sad)
     : [src_stride]"r"((mips_reg)src_stride),
       [ref_stride]"r"((mips_reg)ref_stride)
@@ -742,16 +725,16 @@ static inline unsigned int vpx_sad4x(const uint8_t *src, int src_stride,
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
+    "pxor       %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_REF_ABS_SUB_4
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_REF_ABS_SUB_4
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp3]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
@@ -773,8 +756,6 @@ static inline unsigned int vpx_sad4x(const uint8_t *src, int src_stride,
 
 vpx_sad4xN(8);
 vpx_sad4xN(4);
-sadMxNxK_mmi(4, 4, 3);
-sadMxNxK_mmi(4, 4, 8);
 sadMxNx4D_mmi(4, 8);
 sadMxNx4D_mmi(4, 4);
 
@@ -785,26 +766,27 @@ static inline unsigned int vpx_sad_avg4x(const uint8_t *src, int src_stride,
   unsigned int sad;
   double ftmp1, ftmp2, ftmp3;
   mips_reg l_counter = counter;
+  mips_reg l_second_pred = (mips_reg)second_pred;
 
   /* clang-format off */
   __asm__ volatile (
-    "xor        %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
+    "pxor       %[ftmp3],   %[ftmp3],       %[ftmp3]            \n\t"
     "1:                                                         \n\t"
     // Include two loop body, to reduce loop time.
     SAD_SRC_AVGREF_ABS_SUB_4
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x04            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x04)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
     SAD_SRC_AVGREF_ABS_SUB_4
-    PTR_ADDIU  "%[second_pred], %[second_pred], 0x04            \n\t"
-    PTR_ADDU   "%[src],     %[src],         %[src_stride]       \n\t"
-    PTR_ADDU   "%[ref],     %[ref],         %[ref_stride]       \n\t"
-    PTR_ADDIU  "%[counter], %[counter],     -0x02               \n\t"
+    MMI_ADDIU(%[second_pred], %[second_pred], 0x04)
+    MMI_ADDU(%[src],     %[src],         %[src_stride])
+    MMI_ADDU(%[ref],     %[ref],         %[ref_stride])
+    MMI_ADDIU(%[counter], %[counter], -0x02)
     "bnez       %[counter], 1b                                  \n\t"
     "mfc1       %[sad],     %[ftmp3]                            \n\t"
     : [ftmp1]"=&f"(ftmp1), [ftmp2]"=&f"(ftmp2), [ftmp3]"=&f"(ftmp3),
       [counter]"+&r"(l_counter), [src]"+&r"(src), [ref]"+&r"(ref),
-      [second_pred]"+&r"((mips_reg)second_pred),
+      [second_pred]"+&r"(l_second_pred),
       [sad]"=&r"(sad)
     : [src_stride]"r"((mips_reg)src_stride),
       [ref_stride]"r"((mips_reg)ref_stride)
